/*
 * Copyright (C) 2000  Jesse E. Peterson
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 *
 */

package java.text;

/**
 * Implemented to support unsupported methods within TINI.<p>
 */
public interface CharacterIterator extends Cloneable
{
    /**
     * Constant that is returned when the iterator has reached either the end
     * or the beginning of the text. The unicode 2.0 standard states that
     * '\\uFFFF' is an invalid unicode value and should not occur in any valid
     * unicode string.
     */
    public static final char DONE = '\uFFFF';

    /**
     * Set the position to getBeginIndex() and return the character at that
     * position.
     *
     * @return the first character in the text.
     * @see getBeginIndex
     */
    public abstract char first();

    /**
     * Set the psotion to getEndIndex()-1, return the character at that
     * position.
     *
     * @return the last character in the text.
     * @see getEndIndex
     */
    public abstract char last();

    /**
     * Get the character at the current position (as returned by getIndex()).
     *
     * @return the character at the current position or DONE if the current
     *         position is off the end of the text.
     * @see getIndex
     * 
     */
    public abstract char current();

    /**
     * Increment the iterator's index by one and return the character at the
     * new index. If the resulting index is greater or equal to getEndIndex(),
     * the current index is reset to getEndIndex() and a value of DONE is
     * returned
     *
     * @return the character at the new position or DONE if the current
     *         position is off the end of the text.
     */
    public abstract char next();

    /**
     * Decrement the iterator's index by one and return the character at the
     * new index. If the resulting index isless than getBeginIndex(), the
     * current index is reset to getBeginIndex() and a value of DONE is
     * returned.
     *
     * @return the character at the new position or DONE if the current
     *         position is off the end of the text.
     */
    public abstract char previous();

    /**
     * Set the position to the specified position in the text and return that
     * character.
     *
     * @param position the position within the text. Valid values range from
     *        getBeginIndex() to getEndIndex() - 1. An
     *        IllegalArgumentException is thrown if an invalid value is
     *        supplied.
     * @return the character at the specified position.
     */
    public abstract char setIndex(int position);

    /**
     * Return the start index of the text.
     *
     * @return the index at which the text begins.
     */
    public abstract int getBeginIndex();

    /**
     * Return the end index of the text. This index is the index of the first
     * character following the end of the text.
     *
     * @return the index at which the text end.
     */
    public abstract int getEndIndex();

    /**
     * Return the current index.
     *
     * @return the current index.
     */
    public abstract int getIndex();

    /**
     * Create a copy of this iterator.
     *
     * @return A copy of this.
     */
    public abstract Object clone();
}
