/*
 * Copyright (C) 2000  Jesse E. Peterson
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 *
 */

package java.text;

/**
 * Implemented to support unsupported methods within TINI.<p>
 */
public final class StringCharacterIterator extends Object
    implements CharacterIterator
{
    String text;
    int begin;
    int end;
    int pos;

    /**
     * Construct an iterator with an initial index of 0.
     *
     * @param text The String to be iterated over.
     */
    public StringCharacterIterator(String text)
    {
        this(text, 0);
    }

    /**
     * Construct an iterator with the specified initial index.
     *
     * @param text The String to be iterated over.
     * @param pos Initial iterator position.
     */
    public StringCharacterIterator(String text, int pos)
    {
        this(text, 0, text.length(), 0);
    }

    /**
     * Construct an iterator over the given range of the given string, with
     * the index set at the specified position.
     *
     * @param text The String to be iterated over
     * @param begin Index of the first character
     * @param end Index of the character following the last character
     * @param pos Initial iterator position
     */
    public StringCharacterIterator(String text, int begin, int end, int pos)
    {
        this.text = text;
        this.begin = begin;
        this.end = end;
        this.pos = pos;
    }

    /**
     * Set the position to getBeginIndex() and return the character at that
     * position.
     */
    public char first()
    {
        return(setIndex(getBeginIndex()));
    }

    /**
     * Set the position to getEndIndex() and return the character at that
     * position.
     */
    public char last()
    {
        return(setIndex(getEndIndex()));
    }

    /**
     * Set the position to specified position in the text and return that
     * character.
     */
    public char setIndex(int p)
    {
        pos = p;
        return(current());
    }

    /**
     * Get the character at the current position (as returned by getIndex()).
     *
     * @return the character at the current position or DONE if the current
     *         position is off the end of the text.
     */
    public char current()
    {
        if (pos < getBeginIndex())
        {
            return(DONE);
        }
        if (pos >= getEndIndex())
        {
            return(DONE);
        }
        try
        {
            return(text.charAt(pos));
        }
        catch (StringIndexOutOfBoundsException e)
        {
            return(DONE);
        }
    }

    /**
     * Increment the iterator's index by one and return the character at the
     * new index. If the resulting index is greater or equal to getEndIndex(),
     * the current index is reset to getEndIndex() and a value of DONE is
     * returned.
     *
     * @return the character at the new position or DONE if the current
     *         position is off the end of the text.
     */
    public char next()
    {
        pos++;
        if (pos >= getEndIndex())
        {
            return(last());
        }
        return(current());
    }

    /**
     * Decrement the iterator's index by one and return the character at the
     * new index. If the resulting index is less than getBeginIndex(), the
     * current index is reset to getBeginIndex() and a value of DONE is
     * returned.
     *
     * @return the character at the new position or DONE if the current
     *         position is off the end of the text.
     */
    public char previous()
    {
        pos--;
        if (pos < getBeginIndex())
        {
            first();
            return(DONE);
        }
        return(current());
    }

    /**
     * Return the start index of the text.
     *
     * @return the index at which the text begins.
     */
    public int getBeginIndex()
    {
        return(begin);
    }

    /**
     * Return the end index of the text. This index is the index of the first
     * character following the end of the text.
     *
     * @return the index at which the text ends.
     */
    public int getEndIndex()
    {
        return(end);
    }

    /**
     * Return the current index.
     *
     * @return the current index.
     */
    public int getIndex()
    {
        return(pos);
    }

    /**
     * Compares the equality of two StringCharacterIterator objects.
     *
     * @param obj the StringCharacterIterator object to be compared with.
     * @return true if the given obj is the same as this
     *         StringCharacterItereator object, false otherwise.
     */
    public boolean equals(Object obj)
    {
        if (obj instanceof StringCharacterIterator)
        {
            StringCharacterIterator iterator = (StringCharacterIterator)obj;
            // TODO: Implement this
            System.err.println("StringCharacterIterator.equals() not yet implemented.");
        }
        return(false);
    }

    /**
     * Compute a hashcode for this enumeration.
     *
     * @return A hash code.
     */
    public int hashCode()
    {
        return(text.hashCode());
    }

    /**
     * Create a copy of this.
     *
     * @return A copy of this.
     */
    public Object clone()
    {
        return(new StringCharacterIterator(text, getBeginIndex(),
                                           getEndIndex(), getIndex()));
    }
}
