/*
 * Copyright (C) 1999  Jesse E. Peterson
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 *
 */

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.util.Calendar;
import com.jpeterson.x10.module.CM11A;
import com.jpeterson.x10.module.event.CM11AEvent;
import com.jpeterson.x10.module.event.CM11AListener;
import com.jpeterson.x10.module.event.CM11AStatusEvent;
import com.jpeterson.x10.module.event.CM11AStatusListener;

/**
 * Sample program to set the clock of the CM11A device.
 * <p>
 * This also shows how the CM11A devices status can be retrieved.
 *
 * @author <a href="mailto:jesse@jpeterson.com">Jesse Peterson</a>
 */
public class CM11AEvents implements CM11AListener, CM11AStatusListener
{
    private CM11A cm11A;

    /**
     * @author <a href="mailto:jesse@jpeterson.com">Jesse Peterson</a>
     */
    public static void main(String[] args)
    {
        String port = null;
        int i = 0;
        CM11AEvents cm11AEvents;

        try
        {
            while (i < args.length)
            {
                if (args[i].equals("-port"))
                {
                    // serial port
                    port = args[++i];
                }
                else
                {
                    // error
                    usage();
                    System.exit(1);
                }

                i++;
            }
        }
        catch (ArrayIndexOutOfBoundsException e)
        {
            usage();
            System.exit(1);
        }

        if (port == null)
        {
            cm11AEvents = new CM11AEvents();
        }
        else
        {
            cm11AEvents = new CM11AEvents(port);
        }

        cm11AEvents.listen();
    }

    /**
     * @author <a href="mailto:jesse@jpeterson.com">Jesse Peterson</a>
     */
    public CM11AEvents()
    {
        cm11A = new CM11A();
        try {
            cm11A.allocate();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * @author <a href="mailto:jesse@jpeterson.com">Jesse Peterson</a>
     */
    public CM11AEvents(String port)
    {
        cm11A = new CM11A();
        cm11A.setPortName(port);
        try {
            cm11A.allocate();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Listen for events.
     *
     * @param useCurrentTime True if time should be set to the current time.
     *
     * @author <a href="mailto:jesse@jpeterson.com">Jesse Peterson</a>
     */
    public void listen()
    {
        String command;
        BufferedReader in = new BufferedReader(new InputStreamReader(System.in));

        cm11A.addCM11AListener(this);
        cm11A.addCM11AStatusListener(this);

        while(true)
        {
            System.out.println("Press enter to retrieve CM11A status.");
            System.out.println("Type \"auto\" to auto recover after power failure.");
            System.out.println("Type \"noauto\" to manually recover after power failure.");
            System.out.println("Type \"exit\" to quit.");
            System.out.println("> ");
            command = ""; // initialize value
            try
            {
                command = in.readLine();
            }
            catch (IOException e)
            {
                // in a real program, you would probably want to handle this
            }

            if (command.equals("exit"))
            {
                cm11A.deallocate();
                System.exit(0);
            }
            else if (command.equals("auto"))
            {
                cm11A.setPowerFailureAutoRecover(true);
            }
            else if (command.equals("noauto"))
            {
                cm11A.setPowerFailureAutoRecover(false);
            }
            else
            {
                cm11A.updateStatus();
            }
        }
    }

    /**
     * Implementation of CM11StatusListener. Called after CM11A device
     * detects a power failure. Is called once power is restored.
     *
     * @author <a href="mailto:jesse@jpeterson.com">Jesse Peterson</a>
     */
    public void powerFailure(CM11AEvent e)
    {
        System.out.println("CM11A device detected power failure...");
        CM11A device = (CM11A)e.getSource();

        if (device.getPowerFailureAutoRecover())
        {
            System.out.println("CM11A object will auto recover.");
        }
        else
        {
            System.out.println("Initiating manual recovery.");
            Calendar now = Calendar.getInstance();
            device.setClock(now.get(Calendar.HOUR_OF_DAY),
                            now.get(Calendar.MINUTE),
                            now.get(Calendar.SECOND),
                            now.get(Calendar.MONTH),
                            now.get(Calendar.DAY_OF_MONTH),
                            now.get(Calendar.DAY_OF_WEEK), 'E',
                            false, true, false);
        }
    }

    /**
     * Implementation of CM11StatusListener. Called after CM11A device
     * executes a macro.
     *
     * @author <a href="mailto:jesse@jpeterson.com">Jesse Peterson</a>
     */
    public void macroInitiated(CM11AEvent e)
    {
        System.out.println("CM11A Macro Initiated...");
    }

    /**
     * Implementation of CM11StatusListener.
     *
     * @author <a href="mailto:jesse@jpeterson.com">Jesse Peterson</a>
     */
    public void status(CM11AStatusEvent e)
    {
        System.out.println("Received Status Event...");
        System.out.println(e);
    }

    /**
     * @author <a href="mailto:jesse@jpeterson.com">Jesse Peterson</a>
     */
    public static void usage()
    {
        System.out.println("CM11AEvents [-port serial_port]");
        System.out.println("Where:");
        System.out.println("  -port serial_port  serial_port is name of serial port for the CM11A");
        System.out.println("                     interface. On Windows, \"COM1\" or \"COM2\" for example.");
        System.out.println();
    }
}
