/*
 * Copyright (C) 1999  Jesse E. Peterson
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 *
 */

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Calendar;
import com.jpeterson.x10.Transmitter;
import com.jpeterson.x10.event.AddressEvent;
import com.jpeterson.x10.event.AllLightsOffEvent;
import com.jpeterson.x10.event.AllLightsOnEvent;
import com.jpeterson.x10.event.AllUnitsOffEvent;
import com.jpeterson.x10.event.BrightEvent;
import com.jpeterson.x10.event.DimEvent;
import com.jpeterson.x10.event.OffEvent;
import com.jpeterson.x10.event.OnEvent;
import com.jpeterson.x10.event.X10Event;
import com.jpeterson.x10.module.CM11A;
import com.jpeterson.x10.module.Macro;
import com.jpeterson.x10.module.MacroInitiator;
import com.jpeterson.x10.module.OutOfMacroMemoryException;
import com.jpeterson.x10.module.TimerInitiator;

/**
 * @author Jesse Peterson <jesse@jpeterson.com>
 */
public class DownloadMacros
{
    private CM11A cm11A;
    private String serialFile;

    /**
     * @author Jesse Peterson <jesse@jpeterson.com>
     */
    public static void main(String[] args)
    {
        String port = null;
        String serialFile = null;
        int i = 0;
        DownloadMacros downloadMacros;

        try {
            while (i < args.length) {
                if (args[i].equals("-port"))
                {
                    // serial port
                    port = args[++i];
                }
                else if (args[i].equals("-serialFile"))
                {
                    // CM11A Serialized file
                    serialFile = args[++i];
                }
                else
                {
                    // error
                    usage();
                    System.exit(1);
                }

                i++;
            }
        } catch (ArrayIndexOutOfBoundsException e) {
            usage();
            System.exit(1);
        }

        downloadMacros = new DownloadMacros(serialFile, port);

        downloadMacros.perform();
    }

    /**
     * @author Jesse Peterson <jesse@jpeterson.com>
     */
    public DownloadMacros()
    {
        this(null, null);
    }

    /**
     * @author Jesse Peterson <jesse@jpeterson.com>
     */
    public DownloadMacros(String port)
    {
        this(null, port);
    }

    /**
     * @author Jesse Peterson <jesse@jpeterson.com>
     */
    public DownloadMacros(String serialFile, String port)
    {
        this.serialFile = serialFile;
        allocate(serialFile, port);
    }

    /**
     * @author Jesse Peterson <jesse@jpeterson.com>
     */
    protected void allocate(String serialFile, String port)
    {
        cm11A = null;

        if (serialFile != null)
        {
            // try to deserialize CM11A
            try
            {
                ObjectInputStream istream = new ObjectInputStream(
                    new FileInputStream(serialFile));
                cm11A = (CM11A)istream.readObject();
            }
            catch (IOException e)
            {
                e.printStackTrace();
                System.err.println("Unable to deserialize CM11A from " + serialFile + ", creating new one...");
            }
            catch (ClassNotFoundException e)
            {
                e.printStackTrace();
                System.err.println("Unable to deserialize CM11A from " + serialFile + ", creating new one...");
            }
        }

        if (cm11A == null)
        {
            cm11A = new CM11A();
        }

        if (port != null)
        {
            cm11A.setPortName(port);
        }

        try {
            cm11A.allocate();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Transmit a command
     *
     * @param arg Argument index of command in the command line arguments
     * @param arg Arguments from command line
     *
     * @author Jesse Peterson <jesse@jpeterson.com>
     */
    public void perform()
    {
        MacroInitiator macroInitiator;
        TimerInitiator timerInitiator;
        Macro macro1, macro2;

        // clear any old macros from CM11A interface
        cm11A.clearMacroInitiators();
        cm11A.clearTimerInitiators();

        macro1 = new Macro();
        macro1.addCommand(4, new OnEvent(this, 'E'));
        macro1.addCommand(5, new OnEvent(this, 'E'));

        macro2 = new Macro();
        macro2.addCommand(4, new OffEvent(this, 'E'));
        macro2.addCommand(5, new OffEvent(this, 'E'));

        macroInitiator = new MacroInitiator(7, new OnEvent(this, 'E'));
        macroInitiator.setMacro(macro1);

        cm11A.addMacroInitiator(macroInitiator);

        macroInitiator = new MacroInitiator(7, new OffEvent(this, 'E'));
        macroInitiator.setMacro(macro2);

        cm11A.addMacroInitiator(macroInitiator);

        timerInitiator = new TimerInitiator();
        timerInitiator.addDayOfWeek(Calendar.SUNDAY);
        timerInitiator.addDayOfWeek(Calendar.MONDAY);
        timerInitiator.addDayOfWeek(Calendar.TUESDAY);
        timerInitiator.addDayOfWeek(Calendar.WEDNESDAY);
        timerInitiator.addDayOfWeek(Calendar.THURSDAY);
        timerInitiator.addDayOfWeek(Calendar.FRIDAY);
        timerInitiator.addDayOfWeek(Calendar.SATURDAY);
        timerInitiator.setStartTime(15, 19);
        timerInitiator.setStopTime(15, 20);
        timerInitiator.setStartMacro(macro1);
        timerInitiator.setStopMacro(macro2);

        cm11A.addTimerInitiator(timerInitiator);

        macroInitiator = new MacroInitiator(8, new OffEvent(this, 'E'));
        macro1 = new Macro();
        macro1.addCommand(4, new DimEvent(this, 'E', 11, 22));
        macro1.addCommand(5, new DimEvent(this, 'E', 11, 22));
        macro1.addCommand(9, new OffEvent(this, 'E'));
        macro1.addCommand(6, new OnEvent(this, 'E'));
        macroInitiator.setMacro(macro1);

        cm11A.addMacroInitiator(macroInitiator);

        macroInitiator = new MacroInitiator(9, new OffEvent(this, 'E'));
        macro1 = new Macro(1);
        macro1.addCommand(4, new OffEvent(this, 'E'));
        macro1.addCommand(5, new OffEvent(this, 'E'));
        macroInitiator.setMacro(macro1);

        cm11A.addMacroInitiator(macroInitiator);

        try
        {
            Thread.sleep(3000);
            cm11A.downloadInitiators();
        }
        catch (OutOfMacroMemoryException e)
        {
            e.printStackTrace();
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }

        if (serialFile != null)
        {
            // serialize CM11A for later use
            try
            {
                ObjectOutputStream ostream = new ObjectOutputStream(
                    new FileOutputStream(serialFile));
                ostream.writeObject(cm11A);
                System.out.println("CM11A serialized.");
            }
            catch (IOException e)
            {
                e.printStackTrace();
                System.err.println("Unable to serialize CM11A to " + serialFile + ".");
                System.exit(1);
            }
        }

System.out.print("Deallocating...");
        cm11A.deallocate();
System.out.println("Done");

        System.exit(0);
    }

    /**
     * @author Jesse Peterson <jesse@jpeterson.com>
     */
    public static void usage()
    {
        System.out.println("DownloadMacros [-port serial_port] [-serialFile filename]");
        System.out.println("Where:");
        System.out.println("  -port serial_port  serial_port is name of serial port for the CM11A");
        System.out.println("                     interface. On Windows, \"COM1\" or \"COM2\" for example.");
        System.out.println("  -serialFile filename Once the macros are downloaded, serialize the CM11A");
        System.out.println("                       object out to the file specified by filename.");
        System.out.println();
    }
}
