/*
 * Copyright (C) 1999  Jesse E. Peterson
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 *
 */

import java.io.FileInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import com.jpeterson.x10.Transmitter;
import com.jpeterson.x10.event.AddressEvent;
import com.jpeterson.x10.event.AddressListener;
import com.jpeterson.x10.event.FunctionEvent;
import com.jpeterson.x10.event.FunctionListener;
import com.jpeterson.x10.module.CM11A;

/**
 * @author Jesse Peterson <jesse@jpeterson.com>
 */
public class MonitorX10 implements AddressListener, FunctionListener
{
    private CM11A cm11A;
    private String serialFile;

    /**
     * @author Jesse Peterson <jesse@jpeterson.com>
     */
    public static void main(String[] args)
    {
        String port = null;
        String serialFile = null;
        int i = 0;
        MonitorX10 monitorX10;

        try
        {
            while (i < args.length)
            {
                if (args[i].equals("-port"))
                {
                    // serial port
                    port = args[++i];
                }
                else if (args[i].equals("-serialFile"))
                {
                    // CM11A Serialized file
                    serialFile = args[++i];
                }
                else
                {
                    // error
                    usage();
                    System.exit(1);
                }

                i++;
            }
        }
        catch (ArrayIndexOutOfBoundsException e)
        {
            usage();
            System.exit(1);
        }

        monitorX10 = new MonitorX10(serialFile, port);

        monitorX10.monitor();
    }

    /**
     * @author Jesse Peterson <jesse@jpeterson.com>
     */
    public MonitorX10(String serialFile, String port)
    {
        this.serialFile = serialFile;
        allocate(serialFile, port);
    }

    /**
     * @author Jesse Peterson <jesse@jpeterson.com>
     */
    protected void allocate(String serialFile, String port)
    {
        cm11A = null;

        if (serialFile != null)
        {
            // try to deserialize CM11A
            try
            {
                ObjectInputStream istream = new ObjectInputStream(
                    new FileInputStream(serialFile));
                cm11A = (CM11A)istream.readObject();
            }
            catch (IOException e)
            {
                e.printStackTrace();
                System.err.println("Unable to deserialize CM11A from " + serialFile + ", creating new one...");
            }
            catch (ClassNotFoundException e)
            {
                e.printStackTrace();
                System.err.println("Unable to deserialize CM11A from " + serialFile + ", creating new one...");
            }
        }

        if (cm11A == null)
        {
            cm11A = new CM11A();
        }

        if (port != null)
        {
            cm11A.setPortName(port);
        }

        try {
            cm11A.addAddressListener(this);
            cm11A.addFunctionListener(this);
            cm11A.allocate();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void monitor()
    {
        System.out.println("Listening...");
        try
        {
            cm11A.waitGatewayState(CM11A.DEALLOCATED);
        }
        catch (InterruptedException e)
        {
        }
    }

    /**
     * @author Jesse Peterson <jesse@jpeterson.com>
     */
    public static void usage()
    {
        System.out.println("MonitorX10 [-port serial_port] [-serialFile filename]");
        System.out.println("Where:");
        System.out.println("  -port serial_port  serial_port is name of serial port for the CM11A");
        System.out.println("                     interface. On Windows, \"COM1\" or \"COM2\" for example.");
        System.out.println("  -serialFile filename Once the macros are downloaded, serialize the CM11A");
        System.out.println("                       object out to the file specified by filename.");
        System.out.println();
     }

    public void address(AddressEvent e)
    {
        System.out.println(e);
    }

    public void functionAllUnitsOff(FunctionEvent e)
    {
        System.out.println(e);
    }

    public void functionAllLightsOn(FunctionEvent e)
    {
        System.out.println(e);
    }

    public void functionOn(FunctionEvent e)
    {
        System.out.println(e);
    }

    public void functionOff(FunctionEvent e)
    {
        System.out.println(e);
    }

    public void functionDim(FunctionEvent e)
    {
        System.out.println(e);
    }

    public void functionBright(FunctionEvent e)
    {
        System.out.println(e);
    }

    public void functionAllLightsOff(FunctionEvent e)
    {
        System.out.println(e);
    }

    public void functionHailRequest(FunctionEvent e)
    {
        System.out.println(e);
    }

    public void functionHailAcknowledge(FunctionEvent e)
    {
        System.out.println(e);
    }

    public void functionPresetDim1(FunctionEvent e)
    {
        System.out.println(e);
    }

    public void functionPresetDim2(FunctionEvent e)
    {
        System.out.println(e);
    }

    public void functionStatusOn(FunctionEvent e)
    {
        System.out.println(e);
    }

    public void functionStatusOff(FunctionEvent e)
    {
        System.out.println(e);
    }

    public void functionStatusRequest(FunctionEvent e)
    {
        System.out.println(e);
    }
}

