/*
 * Copyright (C) 1999  Jesse E. Peterson
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 *
 */

import java.io.IOException;
import com.jpeterson.x10.Transmitter;
import com.jpeterson.x10.event.AddressEvent;
import com.jpeterson.x10.event.AllLightsOffEvent;
import com.jpeterson.x10.event.AllLightsOnEvent;
import com.jpeterson.x10.event.AllUnitsOffEvent;
import com.jpeterson.x10.event.BrightEvent;
import com.jpeterson.x10.event.DimEvent;
import com.jpeterson.x10.event.OffEvent;
import com.jpeterson.x10.event.OnEvent;
import com.jpeterson.x10.event.X10Event;
import com.jpeterson.x10.Gateway;
import com.jpeterson.x10.GatewayException;
import com.jpeterson.x10.SerialGateway;

/**
 * @author Jesse Peterson <jesse@jpeterson.com>
 */
public class SendX10
{
    private Transmitter transmitter;

    private static final String DEFAULT_TRANSMITTER = "com.jpeterson.x10.module.CM11A";

    /**
     * @author Jesse Peterson <jesse@jpeterson.com>
     */
    public static void main(String[] args)
    {
        String port = null;
        String transmitter = DEFAULT_TRANSMITTER;
        int i = 0;
        SendX10 sendX10;

        try {
            while (i < args.length) {
                if (args[i].equals("-port"))
                {
                    // serial port
                    port = args[++i];
                }
                else if (args[i].equals("-transmitter"))
                {
                    // transmitter class
                    transmitter = args[++i];
                }
                else
                {
                    // done
                    break;
                }

                i++;
            }
        } catch (ArrayIndexOutOfBoundsException e) {
            usage();
            System.exit(1);
        }

        if (port == null) {
            sendX10 = new SendX10(transmitter);
        } else {
            sendX10 = new SendX10(port, transmitter);
        }

        sendX10.transmit(i, args);
    }

    /**
     * @author Jesse Peterson <jesse@jpeterson.com>
     */
    public SendX10(String transmitter)
    {
        this("COM2", transmitter);
    }

    /**
     * @author Jesse Peterson <jesse@jpeterson.com>
     */
    public SendX10(String port, String transmitterName)
    {
        try
        {
            transmitter = (Transmitter)Class.forName(transmitterName).newInstance();
        }
        catch (ClassNotFoundException e)
        {
            System.err.println("Unknown class: " + transmitterName);
            e.printStackTrace();
            System.exit(1);
        }
        catch (InstantiationException e)
        {
            System.err.println("Error instantiating " + transmitterName);
            e.printStackTrace();
            System.exit(1);
        }
        catch (IllegalAccessException e)
        {
            System.err.println("Illegally accessing " + transmitterName);
            e.printStackTrace();
            System.exit(1);
        }

        if (transmitter instanceof Gateway)
        {
            Gateway gateway = (Gateway)transmitter;
            if ((transmitter instanceof SerialGateway) &&
                (port != null))
            {
                ((SerialGateway)transmitter).setPortName(port);
            }
            try
            {
                gateway.allocate();
            }
            catch (Exception e)
            {
                e.printStackTrace();
            }
        }
    }

    /**
     * Transmit a command
     *
     * @param arg Argument index of command in the command line arguments
     * @param arg Arguments from command line
     *
     * @author Jesse Peterson <jesse@jpeterson.com>
     */
    public void transmit(int arg, String[] args)
    {
        int i = arg;
        X10Event events[] = null;
        String string;
        char houseCode = 0;
        int deviceCode = 0;
        int units = 0;
        System.out.println("command: " + args[i]);
        try {
            if (args[i].equals("on")) {
                // on command
                // houseCode
                string = args[++i];
                if (string == null) {
                    System.err.println("houseCode not specified");
                    usage();
                    System.exit(1);
                }
                houseCode = string.charAt(0);

                // deviceCode
                string = args[++i];
                if (string == null) {
                    System.err.println("deviceCode not specified");
                    usage();
                    System.exit(1);
                }
                try {
                    deviceCode = Integer.parseInt(string);
                } catch (NumberFormatException e) {
                    System.err.println("deviceCode must be a number between 1 and 16");
                    usage();
                    System.exit(1);
                }
                
                events = new X10Event[2];
                events[0] = new AddressEvent(this, houseCode, deviceCode);
                events[1] = new OnEvent(this, houseCode);
            } else if (args[i].equals("off")) {
                System.out.println("Off command");
                // off command
                // houseCode
                string = args[++i];
                if (string == null) {
                    System.err.println("houseCode not specified");
                    usage();
                    System.exit(1);
                }
                houseCode = string.charAt(0);

                // deviceCode
                string = args[++i];
                if (string == null) {
                    System.err.println("deviceCode not specified");
                    usage();
                    System.exit(1);
                }
                try {
                    deviceCode = Integer.parseInt(string);
                } catch (NumberFormatException e) {
                    System.err.println("deviceCode must be a number between 1 and 16");
                    usage();
                    System.exit(1);
                }

                events = new X10Event[2];
                events[0] = new AddressEvent(this, houseCode, deviceCode);
                events[1] = new OffEvent(this, houseCode);
            } else if (args[i].equals("dim")) {
                // dim command
                // houseCode
                string = args[++i];
                if (string == null) {
                    System.err.println("houseCode not specified");
                    usage();
                    System.exit(1);
                }
                houseCode = string.charAt(0);

                // deviceCode
                string = args[++i];
                if (string == null) {
                    System.err.println("deviceCode not specified");
                    usage();
                    System.exit(1);
                }
                try {
                    deviceCode = Integer.parseInt(string);
                } catch (NumberFormatException e) {
                    System.err.println("deviceCode must be a number between 1 and 16");
                    usage();
                    System.exit(1);
                }

                // units
                string = args[++i];
                if (string == null) {
                    System.err.println("units not specified");
                    usage();
                    System.exit(1);
                }
                try {
                    units = Integer.parseInt(string);
                } catch (NumberFormatException e) {
                    System.err.println("units must be a number between 1 and 22");
                    usage();
                    System.exit(1);
                }

                events = new X10Event[2];
                events[0] = new AddressEvent(this, houseCode, deviceCode);
                events[1] = new DimEvent(this, houseCode, units, 22);
            } else if (args[i].equals("bright")) {
                // bright command
                // houseCode
                string = args[++i];
                if (string == null) {
                    System.err.println("houseCode not specified");
                    usage();
                    System.exit(1);
                }
                houseCode = string.charAt(0);

                // deviceCode
                string = args[++i];
                if (string == null) {
                    System.err.println("deviceCode not specified");
                    usage();
                    System.exit(1);
                }
                try {
                    deviceCode = Integer.parseInt(string);
                } catch (NumberFormatException e) {
                    System.err.println("deviceCode must be a number between 1 and 16");
                    usage();
                    System.exit(1);
                }

                // units
                string = args[++i];
                if (string == null) {
                    System.err.println("units not specified");
                    usage();
                    System.exit(1);
                }
                try {
                    units = Integer.parseInt(string);
                } catch (NumberFormatException e) {
                    System.err.println("units must be a number between 1 and 22");
                    usage();
                    System.exit(1);
                }

                events = new X10Event[2];
                events[0] = new AddressEvent(this, houseCode, deviceCode);
                events[1] = new BrightEvent(this, houseCode, units, 22);
            } else if (args[i].equals("allLightsOn")) {
                // all lights on command
                // houseCode
                string = args[++i];
                if (string == null) {
                    System.err.println("houseCode not specified");
                    usage();
                    System.exit(1);
                }
                houseCode = string.charAt(0);

                events = new X10Event[1];
                events[0] = new AllLightsOnEvent(this, houseCode);
            } else if (args[i].equals("allLightsOff")) {
                // all light off command
                // houseCode
                string = args[++i];
                if (string == null) {
                    System.err.println("houseCode not specified");
                    usage();
                    System.exit(1);
                }
                houseCode = string.charAt(0);

                events = new X10Event[1];
                events[0] = new AllLightsOffEvent(this, houseCode);
            } else if (args[i].equals("allUnitsOff")) {
                // all units off command
                // houseCode
                string = args[++i];
                if (string == null) {
                    System.err.println("houseCode not specified");
                    usage();
                    System.exit(1);
                }
                houseCode = string.charAt(0);

                events = new X10Event[1];
                events[0] = new AllUnitsOffEvent(this, houseCode);
            } else {
                // unknown command
                usage();
                System.exit(1);
            }
        } catch (ArrayIndexOutOfBoundsException e) {
            usage();
            System.exit(1);
        }

        if (events == null){
            // shouldn't get here
            usage();
            System.exit(1);
        }

        for (int j = 0; j < events.length; j++)
        {
            System.out.println("Transmitting: " + events[j]);
            try
            {
                transmitter.transmit(events[j]);
            }
            catch (IOException e)
            {
                e.printStackTrace();
            }
        }

        if (transmitter instanceof Gateway)
        {
            Gateway gateway = (Gateway)transmitter;

            try
            {
                System.out.print("Wait for empty queue...");
                gateway.waitGatewayState(Transmitter.QUEUE_EMPTY);
                //cm11A.waitGatewayState(Transmitter.DEALLOCATED);
                System.out.println("Done");
            }
            catch (InterruptedException e)
            {
            }

            System.out.print("Deallocating...");
            try
            {
                gateway.deallocate();
            }
            catch (GatewayException e)
            {
                e.printStackTrace();
            }
            System.out.println("Done");
        }

        System.exit(0);
    }

    /**
     * @author Jesse Peterson <jesse@jpeterson.com>
     */
    public static void usage()
    {
        System.out.println("SendX10 [-port serial_port] [-transmitter class_name] command [options]");
        System.out.println("Where:");
        System.out.println("  -port serial_port  serial_port is name of serial port for the CM11A");
        System.out.println("                     interface. On Windows, \"COM1\" or \"COM2\" for example.");
        System.out.println("  -transmitter class_name Class that implements com.jpeterson.x10.Gateway and");
        System.out.println("                          implements com.jpeterson.x10.module.Transmitter.");
        System.out.println("                          Default: " + DEFAULT_TRANSMITTER);
        System.out.println("  command            Any one of the commands described below.");
        System.out.println("  options            Any options for the specified command.");
        System.out.println();
        System.out.println("Commands:");
        System.out.println("  on houseCode deviceCode");
        System.out.println("  off houseCode deviceCode");
        System.out.println("  dim houseCode deviceCode units");
        System.out.println("  bright houseCode deviceCode units");
        System.out.println("  allLightsOn houseCode");
        System.out.println("  allLightsOff houseCode");
        System.out.println("  allUnitsOff houseCode");
        System.out.println("Where:");
        System.out.println("  houseCode  The house code 'A' through 'P', uppercase.");
        System.out.println("  deviceCode The device code '1' through '16'.");
        System.out.println("  units      Number of units to dim or brighten.");
    }
}
