/*
 * Copyright (C) 1999  Jesse E. Peterson
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 *
 */

import java.util.Calendar;
import com.jpeterson.x10.Transmitter;
import com.jpeterson.x10.event.AddressEvent;
import com.jpeterson.x10.event.AllLightsOffEvent;
import com.jpeterson.x10.event.AllLightsOnEvent;
import com.jpeterson.x10.event.AllUnitsOffEvent;
import com.jpeterson.x10.event.BrightEvent;
import com.jpeterson.x10.event.DimEvent;
import com.jpeterson.x10.event.OffEvent;
import com.jpeterson.x10.event.OnEvent;
import com.jpeterson.x10.event.X10Event;
import com.jpeterson.x10.module.CM11A;
import com.jpeterson.x10.module.Macro;
import com.jpeterson.x10.module.MacroInitiator;
import com.jpeterson.x10.module.OutOfMacroMemoryException;
import com.jpeterson.x10.module.TimerInitiator;
import com.jpeterson.x10.module.event.CM11AStatusEvent;
import com.jpeterson.x10.module.event.CM11AStatusListener;

/**
 * Sample program to set the clock of the CM11A device.
 * <p>
 * This also shows how the CM11A devices status can be retrieved.
 *
 * @author <a href="mailto:jesse@jpeterson.com">Jesse Peterson</a>
 */
public class SetClock implements CM11AStatusListener
{
    private CM11A cm11A;

    /**
     * @author <a href="mailto:jesse@jpeterson.com">Jesse Peterson</a>
     */
    public static void main(String[] args)
    {
        String port = null;
        int i = 0;
        SetClock setClock;
        boolean now = false;

        try
        {
            while (i < args.length)
            {
                if (args[i].equals("-port"))
                {
                    // serial port
                    port = args[++i];
                }
                else if (args[i].equals("-now"))
                {
                    // set time to current time
                    now = true;
                }
                else
                {
                    // error
                    usage();
                    System.exit(1);
                }

                i++;
            }
        }
        catch (ArrayIndexOutOfBoundsException e)
        {
            usage();
            System.exit(1);
        }

        if (port == null)
        {
            setClock = new SetClock();
        }
        else
        {
            setClock = new SetClock(port);
        }

        setClock.set(now);
    }

    /**
     * @author <a href="mailto:jesse@jpeterson.com">Jesse Peterson</a>
     */
    public SetClock()
    {
        cm11A = new CM11A();
        try {
            cm11A.allocate();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * @author <a href="mailto:jesse@jpeterson.com">Jesse Peterson</a>
     */
    public SetClock(String port)
    {
        cm11A = new CM11A();
        cm11A.setPortName(port);
        try {
            cm11A.allocate();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Set the CM11A clock.
     *
     * @param useCurrentTime True if time should be set to the current time.
     *
     * @author <a href="mailto:jesse@jpeterson.com">Jesse Peterson</a>
     */
    public void set(boolean useCurrentTime)
    {
        try
        {
            Thread.sleep(3000);
            if (useCurrentTime)
            {
                Calendar now = Calendar.getInstance();
                cm11A.setClock(now.get(Calendar.HOUR_OF_DAY),
                               now.get(Calendar.MINUTE),
                               now.get(Calendar.SECOND),
                               now.get(Calendar.MONTH),
                               now.get(Calendar.DAY_OF_MONTH),
                               now.get(Calendar.DAY_OF_WEEK), 'E',
                               true, true, true);
            }
            else
            {
                cm11A.setClock(23, 58, 45, Calendar.FEBRUARY, 28,
                               Calendar.MONDAY, 'E', false, true, false);
            }

            // call updateStatus to verify that the clock was set.
            cm11A.updateStatus(this);

            // wait for status response.
            // This is a quick way of waiting for demonstration purposes
            // only.
            Thread.sleep(3000);

        }
        catch (Exception e)
        {
            e.printStackTrace();
        }

System.out.print("Deallocating...");
        cm11A.deallocate();
System.out.println("Done");

        System.exit(0);
    }

    /**
     * Implementation of CM11StatusListener.
     *
     * @author <a href="mailto:jesse@jpeterson.com">Jesse Peterson</a>
     */
    public void status(CM11AStatusEvent e)
    {
        System.out.println("Status:");
        System.out.println(e);
    }

    /**
     * @author <a href="mailto:jesse@jpeterson.com">Jesse Peterson</a>
     */
    public static void usage()
    {
        System.out.println("SetClock [-port serial_port] [-now]");
        System.out.println("Where:");
        System.out.println("  -port serial_port  serial_port is name of serial port for the CM11A");
        System.out.println("                     interface. On Windows, \"COM1\" or \"COM2\" for example.");
        System.out.println("  -now  Set the clock to the current time.");
        System.out.println();
    }
}
