/*
 * Copyright (C) 1999  Jesse E. Peterson
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 *
 */

import javax.speech.*;
import javax.speech.synthesis.*;
import com.jpeterson.x10.Transmitter;
import com.jpeterson.x10.event.AddressEvent;
import com.jpeterson.x10.event.AddressListener;
import com.jpeterson.x10.event.FunctionEvent;
import com.jpeterson.x10.event.FunctionListener;
import com.jpeterson.x10.module.CM11A;

/**
 * @author Jesse Peterson <jesse@jpeterson.com>
 */
public class SpeakX10 implements AddressListener, FunctionListener
{
    private CM11A cm11A;
    Synthesizer synth;
    static String modeName;

    /**
     * @author Jesse Peterson <jesse@jpeterson.com>
     */
    public static void main(String[] args)
    {
	String port = null;
	int i = 0;
	SpeakX10 speakX10;

	try {
	    while (i < args.length) {
		if (args[i].equals("-port")) {
		    // serial port
		    port = args[++i];
		} else if (args[i].equals("-voice")) {
		    // mode name
		    modeName = args[++i];
		} else {
		    // done
		    break;
		}

		i++;
	    }
	} catch (ArrayIndexOutOfBoundsException e) {
	    usage();
	    System.exit(1);
	}

	if (port == null) {
	    speakX10 = new SpeakX10();
	} else {
	    speakX10 = new SpeakX10(port);
	}

	speakX10.monitor();
    }

    /**
     * @author Jesse Peterson <jesse@jpeterson.com>
     */
    public SpeakX10()
    {
	this("COM2");
    }

    /**
     * @author Jesse Peterson <jesse@jpeterson.com>
     */
    public SpeakX10(String port)
    {
	cm11A = new CM11A();
	cm11A.setPortName(port);

	try {
	    SynthesizerModeDesc mode = null;
	    if (modeName != null) {
		SynthesizerModeDesc desc = new SynthesizerModeDesc();
		desc.setModeName(modeName);
		EngineList list = Central.availableSynthesizers(desc);
		mode = (SynthesizerModeDesc)list.elementAt(0);
	    }
	    synth = Central.createSynthesizer(mode);
	    synth.allocate();
	} catch (EngineException e) {
	    System.err.println("Caught EngineExcepiton while creating synthesizer.");
	    e.printStackTrace();
	    System.exit(1);
	}

	try {
	    cm11A.addAddressListener(this);
	    cm11A.addFunctionListener(this);
	    cm11A.allocate();
	} catch (Exception e) {
	    System.err.println("Unable to allocate CM11A interface.");
	    e.printStackTrace();
	    System.exit(1);
	}
    }

    public void monitor()
    {
	System.out.println("Listening...");
	try {
	    cm11A.waitGatewayState(CM11A.DEALLOCATED);
	} catch (InterruptedException e) {
	}
    }

    /**
     * @author Jesse Peterson <jesse@jpeterson.com>
     */
    public static void usage()
    {
	System.out.println("SpeakX10 [-port serial_port] [-voice mode_name]");
	System.out.println("Where:");
	System.out.println("  -port serial_port  serial_port is name of serial port for the CM11A");
	System.out.println("                     interface. On Windows, \"COM1\" or \"COM2\" for example.");
	System.out.println("  -voice mode_name   optional mode name for voice synthesizer");
    }

    public void address(AddressEvent e)
    {
	synth.speakPlainText("Received address event for house code '" + 
			     e.getHouseCode() +
			     "', device code " +
			     e.getDeviceCode() +
			     ".", null);
    }

    public void functionAllUnitsOff(FunctionEvent e)
    {
	synth.speakPlainText("Received 'All Units Off' event for house code '" + 
			     e.getHouseCode() +
			     "'.", null);
    }

    public void functionAllLightsOn(FunctionEvent e)
    {
	synth.speakPlainText("Received 'All Lights On' event for house code '" + 
			     e.getHouseCode() +
			     "'.", null);
    }

    public void functionOn(FunctionEvent e)
    {
	synth.speakPlainText("Received 'On' event for house code '" + 
			     e.getHouseCode() +
			     "'.", null);
    }

    public void functionOff(FunctionEvent e)
    {
	synth.speakPlainText("Received 'Off' event for house code '" + 
			     e.getHouseCode() +
			     "'.", null);
    }

    public void functionDim(FunctionEvent e)
    {
	synth.speakPlainText("Received 'Dim' event for house code '" + 
			     e.getHouseCode() +
			     "'.", null);
    }

    public void functionBright(FunctionEvent e)
    {
	synth.speakPlainText("Received 'Bright' event for house code '" + 
			     e.getHouseCode() +
			     "'.", null);
    }

    public void functionAllLightsOff(FunctionEvent e)
    {
	synth.speakPlainText("Received 'All Lights Off' event for house code '" + 
			     e.getHouseCode() +
			     "'.", null);
    }

    public void functionHailRequest(FunctionEvent e)
    {
	synth.speakPlainText("Received 'Hail Request' event for house code '" + 
			     e.getHouseCode() +
			     "'.", null);
    }

    public void functionHailAcknowledge(FunctionEvent e)
    {
	synth.speakPlainText("Received 'Hail Acknowledge' event for house code '" + 
			     e.getHouseCode() +
			     "'.", null);
    }

    public void functionPresetDim1(FunctionEvent e)
    {
	synth.speakPlainText("Received 'Preset Dim 1' event for house code '" + 
			     e.getHouseCode() +
			     "'.", null);
    }

    public void functionPresetDim2(FunctionEvent e)
    {
	synth.speakPlainText("Received 'Preset Dim 2' event for house code '" + 
			     e.getHouseCode() +
			     "'.", null);
    }

    public void functionStatusOn(FunctionEvent e)
    {
	synth.speakPlainText("Received 'Status On' event for house code '" + 
			     e.getHouseCode() +
			     "'.", null);
    }

    public void functionStatusOff(FunctionEvent e)
    {
	synth.speakPlainText("Received 'Status Off' event for house code '" + 
			     e.getHouseCode() +
			     "'.", null);
    }

    public void functionStatusRequest(FunctionEvent e)
    {
	synth.speakPlainText("Received 'Status Request' event for house code '" + 
			     e.getHouseCode() +
			     "'.", null);
    }
}

