/*
 * Copyright (C) 1999  Jesse E. Peterson
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 *
 */

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.net.Socket;
import java.net.UnknownHostException;
import com.jpeterson.x10server.X10ServerSocketObjectPool;

/**
 * Example client that talks with the X10Server via a TCP socket. This client
 * also uses the X10ServerSocketObjectPool to handle the connections to the
 * X10Server.
 * <P>
 * This program will prompt for X10Server commands with a '$' prompt. When you
 * receive the prompt, enter an X10Server command. For example, to turn on
 * device 8 on house code 'C', enter "(C08)(CON)" followed by a carriage
 * return.
 * <P>
 * When running this demo client, the first argument is the host name or IP
 * address of the X10Server. The second argument is the port that the
 * X10Server is listening on. The second argument, the port, is optional. If
 * not provided, the default port of 4377 is used.
 *
 * @author Jesse Peterson <jesse@jpeterson.com>
 */
public class X10ServerClient
{
    public static final String PROMPT = ">";
    X10ServerSocketObjectPool x10pool;

    /**
     * Initialize and start the client.
     *
     * @param args The first argument is the host name or IP address of the
     *        X10Server and is required. The second argument is the port
     *        number that the X10Server is listening on. If the port number
     *        is not defined, the default value of 4377 is used.
     */
    public static void main(String[] args)
    {
        int port;

        try
        {
            if (args.length > 1)
            {
                port = Integer.parseInt(args[1]);
            }
            else
            {
                port = 4377;
            }
        }
        catch (NumberFormatException e)
        {
            port = 4377;
        }

        try
        {
            X10ServerClient client = new X10ServerClient(args[0], port);
            client.service();
        }
        catch (UnknownHostException e)
        {
            e.printStackTrace();
        }
    }

    /**
     * Create an X10ServerClient.
     *
     * @param hostname Host name or IP address where the X10Server is running.
     * @param port Port that the X10Server is listening for connections on.
     * @exception UnknownHostException The parameter hostname is invalid.
     *
     * @author Jesse Peterson <jesse@jpeterson.com>
     */
    public X10ServerClient(String hostname, int port) throws UnknownHostException
    {
        x10pool = new X10ServerSocketObjectPool(hostname, port);
    }

    /**
     * Provide the user interface for the client. Command line driven. Prompts
     * with a '$' character when ready for input.
     */
    public void service()
    {
        String line;
        StringBuffer buf;
        Socket socket;
        BufferedReader in = new BufferedReader(new InputStreamReader(
                                System.in));
        BufferedInputStream x10rsp;
        Writer x10req;
        byte[] bytes = new byte[80];
        int count;
        String response, error;

        buf = new StringBuffer();

        try
        {
            // print prompt
            System.out.print("$");
            for (line = in.readLine(); !line.toUpperCase().equals("QUIT"); line = in.readLine())
            {
                socket = (Socket)x10pool.borrowObject();

                if (socket == null)
                {
                    System.err.println("Unable to get a free connection.");

                    // print prompt
                    System.out.print("$");
                    continue;
                }

                try
                {
                    buf.setLength(0); // clear the buffer

                    x10rsp = new BufferedInputStream(socket.getInputStream());
                    x10req = new OutputStreamWriter(socket.getOutputStream());

                    // send request
                    x10req.write(line + "\n");
                    x10req.flush();

                    // read response
                    do
                    {
                        count = x10rsp.read(bytes);

                        if (count < 0)
                        {
                            // end of connection
                            x10pool.returnBrokenObject(socket);
                            socket = null;
                            return;
                        }
                        if (count > 0)
                        {
                            buf.append(new String(bytes, 0, count));
                        }
                    } while ((buf.toString()).indexOf(PROMPT) == -1);
                    response = buf.toString();
                    if (response.equals(PROMPT))
                    {
                        // no error
                    }
                    else
                    {
                        error = response.substring(0, response.indexOf(PROMPT)).trim();
                        System.err.println("Error: " + error);
                    }

                    // print prompt
                    System.out.print("$");
                }
                catch (IOException e)
                {
                    x10pool.returnBrokenObject(socket);
                    socket = null;
                }
                finally
                {
                    if (socket != null)
                    {
                        x10pool.returnObject(socket);
                    }
                }
            }
        }
        catch (IOException ex)
        {
            ex.printStackTrace();
        }
    }
}
