/*
 * Copyright (C) 2000  Jesse E. Peterson
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 *
 */

import java.io.IOException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import com.jpeterson.x10.Transmitter;
import com.jpeterson.x10.embedded.CM11AGateway;
import com.jpeterson.x10server.X10ServerStub;

public class EmbeddedGateway extends X10ServerStub
{
    /**
     * X10 gateway.
     */
    protected CM11AGateway gateway;

    public static void main(String[] args)
    {
        System.out.println("Executing EmbeddedGateway");

        int listenPort;
        InetAddress statusAddress = null;
        int statusPort;
        String comPort;

        // set default values
        listenPort = DEFAULT_LISTEN_PORT;
        try
        {
            statusAddress = InetAddress.getByName(DEFAULT_STATUS_ADDRESS);
        }
        catch (UnknownHostException e)
        {
            e.printStackTrace();
            System.exit(EXIT_INVALID_LISTEN_PORT);
        }
        statusPort = DEFAULT_STATUS_PORT;
        comPort = DEFAULT_COM_PORT;

        try
        {
            for (int i = 0; i < args.length; i++)
            {
                if (args[i].equals("-listenport"))
                {
                    try
                    {
                        listenPort = Integer.parseInt(args[++i]);
                    }
                    catch (NumberFormatException e)
                    {
                        System.err.println("Invalid listen port number.");
                        usage();
                        System.exit(EXIT_INVALID_LISTEN_PORT);
                    }
                }
                else if (args[i].equals("-statusaddress"))
                {
                    try
                    {
                        statusAddress = InetAddress.getByName(args[++i]);
                    }
                    catch (UnknownHostException e)
                    {
                        System.err.println("Invalid status address.");
                        usage();
                        System.exit(EXIT_INVALID_STATUS_ADDRESS);
                    }
                }
                else if (args[i].equals("-statusport"))
                {
                    try
                    {
                        statusPort = Integer.parseInt(args[++i]);
                    }
                    catch (NumberFormatException e)
                    {
                        System.err.println("Invalid listen port number.");
                        usage();
                        System.exit(EXIT_INVALID_LISTEN_PORT);
                    }
                }
                else if (args[i].equals("-port"))
                {
                    comPort = args[++i];
                }
                else
                {
                    usage();
                    System.exit(EXIT_INVALID_ARGUMENT);
                }
            }
        }
        catch (ArrayIndexOutOfBoundsException e)
        {
            usage();
            System.exit(EXIT_INVALID_ARGUMENT);
        }

        try
        {
            EmbeddedGateway gateway = new EmbeddedGateway(listenPort,
                                                          statusAddress,
                                                          statusPort,
                                                          comPort);

            gateway.start();
        }
        catch (UnknownHostException e)
        {
            e.printStackTrace();
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
    }

    /**
     * Print out usage information.
     *
     * @author Jesse Peterson <jesse@jpeterson.com>
     */
    public static void usage()
    {
        System.out.println("EmbeddedGateway [-listenport port] [-statusaddress IPAddress] [-statusport port] [-com comport]");
        System.out.println("where:");
        System.out.println("  -listenport port");
        System.out.println("     Set the port that the server listens for control to connect to.");
        System.out.println("     Default: " + DEFAULT_LISTEN_PORT);
        System.out.println();
        System.out.println("  -statusaddress IPAddress");
        System.out.println("     Set the status broadcast address that status messages will be sent on.");
        System.out.println("     Must be a valid multicast address from 224.0.0.1 through 239.255.255.255,");
        System.out.println("     inclusive.");
        System.out.println("     Default: " + DEFAULT_STATUS_ADDRESS);
        System.out.println();
        System.out.println(" -statusport port");
        System.out.println("     Set the port that the status messages will be sent to.");
        System.out.println("     Default: " + DEFAULT_STATUS_PORT);
        System.out.println();
        System.out.println(" -port comport");
        System.out.println("     Set the com port that the CM11A can be contacted on.");
        System.out.println("     Default: " + DEFAULT_COM_PORT);
    }

    /**
     * Create a new X10Server.
     *
     * @param serialFile Filename of serialized CM11A.
     * @param listenPort TCP port to listen for client connections on.
     * @param statusAddress UDP multicast address to send status messages
     *        to.
     * @param statusPort UDP port to send status messages to.
     * @param comPort Serial port to talk to CM11A to.
     * @author Jesse Peterson <jesse@jpeterson.com>
     */
    public EmbeddedGateway(int listenPort,
                     InetAddress statusAddress, int statusPort,
                     String comPort)
    {
        super(listenPort, statusAddress, statusPort, comPort);
        System.out.println("Listen port = " + listenPort);
        System.out.println("Status address = " + statusAddress);
        System.out.println("Status port = " + statusPort);
        System.out.println("Serial port = " + comPort);
    }

    /**
     * Create an instance of an <code>CM11AGateway</code> object, which
     * implements the <code>Transmitter</code> interface.
     */
    protected Transmitter createTransmitter()
    {
        gateway = new CM11AGateway(comPort);

        gateway.addAddressListener(this);
        gateway.addFunctionListener(this);

        Thread t = new Thread(gateway);
        t.start();

        return(gateway);
    }
}
