/*
 * Copyright (C) 2000  Jesse E. Peterson
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 *
 */

package java.text;

import java.io.Serializable;

/**
 * Implemented to support unsupported methods within TINI.
 */
public abstract class Format extends Object implements Serializable, Cloneable
{
    public Format()
    {
    }

    /**
     * Formats an object to produce a string.<P>
     * Subclasses will override the StringBuffer version of format.
     *
     * @param obj The object to format
     * @return Formatted String
     * @excpetion IllegalAgumentException when the Format cannot format the
     *            type of object.
     */
    public final String format(Object obj)
    {
        return format(obj, new StringBuffer(), new FieldPosition(0)).toString();
    }

    /**
     * Formats an object to produce a string. Subclasses will implement
     * for particular object, such as:
     * <blockquote><code>
     * StringBuffer format (Number obj, StringBuffer toAppendTo)<br>
     * Number parse (String str)
     * </code></blockquote>
     * These general routines allow polymorphic parsing and formatting for
     * objects such as the MessageFormat.
     *
     * @param obj The object to format
     * @param toAppendTo where the text is to be appended
     * @param status On input: an alignment field, if desired. On output:
     *        the offsets of the alignment field.
     * @return the value passed in as toAppendTo (this allows chaining,
     *         as with StringBuffer.append()
     * @exception IllegalArgumentException when the Format cannot format
     *            the given object.
     */
    public abstract StringBuffer format(Object obj, StringBuffer toAppendTo,
                                        FieldPosition pos);

    /**
     * Parses a string to produce an object. Subclasses will typically
     * implement for particular object, such as:
     * <blockquote><code>
     * String format (Number obj);<br>
     * String format (long obj);<br>
     * String format (double obj);<br>
     * Number parse (String str);<br>
     * </code></blockquote>
     *
     * @param parsePostion - Input-Output parameter.<p>
     * Before calling, set status.index to to offset you want to start parsing
     * at in the source. After calling, status.index is the end of the text
     * you parsed. If error occurs, index is unchanged.<p>
     * When parsing, leading whitespace is discarded (with successful parse),
     * while trailing whitespace is left as is.<p>
     * Example: Parsing "_12_xy" (where _ represents a space) for a number,
     * with index == 0 will result in the number 12, with status.index updated
     * to 3 (just before the second space). Parsing a second time will result
     * in a ParseException since "xy" is not a number, and leave index at 3.<p>
     * Subclasses will typically supply specific parse methods that return
     * different types of values. Since methods can't overload on return
     * types, these will typically be named "parse", while this polymorphic
     * method will always be called parseObject. Any parse method that does
     * not take a status should throw ParseException when no text in the
     * required format is at the start position.
     * @return Object parse from string. In case of error, returns null.
     *
     */
    public abstract Object parseObject(String source, ParsePosition status);

    /**
     * Parses a string to produce an object.
     *
     * @exception ParseException if the specified string is invalid.
     */
    public Object parseObject(String source) throws ParseException
    {
        return(parseObject(source, new ParsePosition(0)));
    }

    /**
     * Create a new object of the same class as this object.
     */
    public Object clone()
    {
        return(null);
    }
}
